#include "mad.h"
#include "prop.h"

/*
     String Escaper - used for determining string escapes,
   such as 'Hello\tSir,\nWelcome\n".
*/

class StringEscaper{
protected:
   const char  *s;
   unsigned int pos;
   boolean      state;

   char getChar_();
public:
   StringEscaper(const char *s):s(s),
                          pos(0),
			  state(false){};
			  
   virtual ~StringEscaper(){};
   
   char getChar();

   //This function must be used for transforming escapes,
   //such as 'n' --> '\n' (new_line), or 't' --> '\t' (tab)
   virtual char returnEscape(char x){ return x; };

   inline boolean getStatus(){ return state; };
   inline boolean eof(){ return pos >= strlen(s); };
};

/*
     Macro Replacer - replace 'macroses' into strings
   by Nikolay Mijaylov  (c) 1996(?), 1998, 1999, Sofia

   first realisation - Turbo Pascal, non objects

   object tree:
   
   AbstractMacroReplacer(8)=+-----MacroReplacer
*/

class AbstractMacroReplacer{
protected:
   StringBuffer *sb;
   
   char          bdelim;
   char          edelim;
   
   short int     logic_status;
   
   void initLogic();
   char logic(char x);
   
public:
   AbstractMacroReplacer(char bdelim = '[', char edelim = ']'):
                                sb(new StringBuffer()),
                                bdelim(bdelim),
                                edelim(edelim){};
   virtual ~AbstractMacroReplacer(){
      delete(sb);
   };

   const char *replace(const char *source);

   virtual const char *replaceField(const char *) = 0;
};

class MacroReplacer:public AbstractMacroReplacer{
protected:
   Properties *pr;
public:
   MacroReplacer(Properties *pr, char bdelim = '[', char edelim = ']'):
                                AbstractMacroReplacer(bdelim, edelim),
                                pr(pr){
   };
   const char *replaceField(const char *field);
};
