/*
 * Standrd library with many functions for:
 *    1. DOS/Win/UNIX compatibility
 *    2. NULL Strings.
 *    3. boolean type :)
 *    4. String Buffer
 *    5. Java style string tokenizer
 *
 * CopyLeft (c) 1998,99 by Nikolay Mijaylov
 *             "www.nmmm.nu" <nmmm@nmmm.nu>
 */
#ifndef _MAD_MAD_H
#define _MAD_MAD_H

#ifndef boolean
#define boolean int
#endif

#ifndef true
#define true    1
#endif

#ifndef false
#define false   0
#endif

#ifdef __DOS__
#define PATH_DELIMITER_STR "\\"
#else
#define PATH_DELIMITER_STR "/"
#endif

#define PATH_DELIMITER     PATH_DELIMITER_STR[0]

#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#ifndef __BORLANDC__
//strcasecmp is avail in BSD and GCC, how to fix it ???
#define stricmp strcasecmp
#endif

inline void printferr(const char *s){
   fprintf(stderr, s); };
inline boolean isempty(const char *c){
   return (c == NULL)
 ? true : (strlen(c) == 0);
};
inline char *strdupn(const char *c){
   return (isempty(c))
 ? NULL : strdup(c);};
inline int  strcmpn(const char *c1, const char *c2){
   return (!c1 || !c2) ? 1 : strcmp(c1, c2);}
;
inline int  stricmpn(const char *c1, const char *c2){
   return (!c1 || !c2) ? 1 : stricmp(c1, c2);}
;

char *to_upper(char *c);
char *to_lower(char *c);

char *strcut(char *c);
long int file2char(const char *filename, char *buffer, long int size);
char *filegets(char *buffer, int max, FILE *f);
void replacechar(char *buffer, char c1, char c2);

//Don't smile :)))))))))))))
//I know how to fix it, but can you tell me WHY?!?!?!:)
char *strconcat(const char *x1 = NULL,
                const char *x2 = NULL,
                const char *x3 = NULL,
                const char *x4 = NULL,
                const char *x5 = NULL);

inline int atoid(const char *
s, int i){
   return isempty(s) ? i : atoi(s);};
//This may be works under BC++
char *itoa(int value);
//itoa with buffer, often for internal use of itoa...
char *itoa(int value, char *buff); //Buff is max 15 chars :)

int setenv(const char *name, const char *value, char *buffer);

class StringBuffer{
private:
   char    *data;
   boolean destroydata;
public:
   StringBuffer(boolean destroydata = true):data(strdup("")),
                                    destroydata(destroydata){};
   virtual ~StringBuffer(){
      if (destroydata)
         free(data);
   };
   void clear(){
      free(data);
      data = strdup("");
   };
   void add(const char c);
   void add(const char *c);
   const char *get(){
      return data;
   };
};

class StringTokenizer{
private:
   char *data;
   char delim;
   int  index;
   int  len;
public:
   //U can;t construct StringTokenizer with a NULL Pointer !!!
   StringTokenizer(const char *data, char delim):data(strdup(data)),
                                                 delim(delim),
                                                 index(0),
                                                 len(strlen(this->data)){};
   virtual ~StringTokenizer(){
      free(data);
   };
   const char *nextToken();
   const char *nextToEnd();
};

#endif
//_MAD_MAD_H