#include <string.h>
#include <ctype.h>

#include "../mad/mad.h"

#include "xml_parser.h"

#define  OPENTAG  '<'
#define  CLOSETAG '>'
#define  OFFTAG   "/" /*must be string*/

boolean XMLParser::loadFromFile(const char *filename){
   pos=0;
   len = file2char(filename, data, maxlen);
   return (len >= 0);
};

boolean XMLParser::loadFromString(const char *s){
   if (isempty(s))
      return false;

   //It;s faster, isn't it?
   len=strlen(s); if (len > maxlen) len = maxlen;

   strncpy(data, s, len);
   data[len] = '\n';
   pos=0;
   return true;
};

boolean XMLParser::eof(){
   return (pos >= len);
}

const char *XMLParser::seekChar(char ch){
   if (eof())
      return NULL;

   long int pos1 = pos;
   long int i;
   for (i=pos; data[i]!='\0' && data[i]!=ch; i++);
   pos=i;

   data[pos]='\0';
   return &data[pos1];
};

const char *XMLParser::seekNextTag(){
   return seekChar(OPENTAG);
};

const char *XMLParser::seekNextTag(char *m[]){
   doublebuffer->clear();
   boolean f = true;

   while (f && !eof()){
      doublebuffer->add(seekNextTag());

      long int pos1 = pos;
      const char *s = getNextTag();

      for(unsigned int i = 0; m[i]; i++)
         if (cmpTag(s, m[i]) > 0){
            f = false;
            pos =  pos1;
            break;
         }

      if (f && !isempty(s)){
         doublebuffer->add(OPENTAG);
         doublebuffer->add(s);
         doublebuffer->add(CLOSETAG);
      }
   }
   return doublebuffer->get();
};

const char *XMLParser::getNextTag(){
   //data[pos]=OPENTAG;
   pos++;
   const char *s = seekChar(CLOSETAG);
   pos++;
   return s;
};

char *XMLParser::cutTag(char *s){
   if (isempty(s))
      return NULL;

   boolean f = false;

   unsigned int i;

   for (i=0; i < strlen(s); i++)
      if (!isspace(s[i]))
         if (s[i] == OFFTAG[0])
            f=true;
         else
            break;

   if (f){
      i--;
      s[i]=OFFTAG[0]; //No problem :)
   }
   
   unsigned int j;
   for (j=i; j < strlen(s); j++)
      if (isspace(s[j])) break;

   s[j]='\0';

   return &s[i];
};

int XMLParser::cmpTag(const char *tag1, const char *tag2){
   char *tag_1=strdupn(tag1);
   char *tag_2=strdupn(tag2);

   char *t1   =cutTag(tag_1);
   char *t2   =cutTag(tag_2);
   char *t1_  =strconcat(OFFTAG, t1);

   int x=0;
   if (!stricmpn(t1, t2)) 
      x++;
   else if (!strcmpn(t1_, t2))
      x--;

   //printf("\n%x>>>%s-%s-%s<<<\n", x, t1, t1_, t2);
   //read();

   free(tag_1);
   free(tag_2);
   free(t1_);

   return x;
};

const char *XMLParser::extractDoubleTag(char *tag){
   doublebuffer->clear();

   const char *tag1=NULL;
   int   f=0;

   while ((f >= 0) && 
          (!eof())){
      if (!isempty(tag1)){
         doublebuffer->add(OPENTAG);
         doublebuffer->add(tag1);
         doublebuffer->add(CLOSETAG);
      }
      doublebuffer->add(seekNextTag());
      tag1=getNextTag();
      f=f+cmpTag(tag, tag1);
   }

   return doublebuffer->get();
};
